/*
 * T.C.F.S. Utils 2.0 $Revision: 2.0 $
 *
 * Authors:	Giuseppe Cattaneo, <cattaneo@udsab.dia.unisa.it>
 *		Giuseppe Persiano, <giuper@udsab.dia.unisa.it>
 *		Luigi Catuogno, <luicat@mikonos.dia.unisa.it>
 *		Andrea Cozzolino, <andcoz@mikonos.dia.unisa.it>
 *		Angelo Celentano, <angcel@mikonos.dia.unisa.it>
 *		Aniello Del Sorbo, <anidel@mikonos.dia.unisa.it>
 *		Ermelindo Mauriello, <ermmau@mikonos.dia.unisa.it>
 *		Raffaele Pisapia, <rafpis@mikonos.dia.unisa.it>
 *
 * Permission  to use,  copy,  and modify this  software  without fee
 * is hereby granted, provided that this entire notice is included in
 * all  copies  of  any  software  which  is  or includes  a copy  or
 * modification of this  software and in all copies of the supporting
 * documentation for such software.
 *
 * This  software maybe  be used for  any purpose provided  the above 
 * copyright  notice  is  retained.  It is  supplied as  is, with  no 
 * warranty expressed or implied.
 */

/*
 *       $Source: /home/anidel/Wip/TCFS-2.2.0a/tcfslib-0.2/include/RCS/config.h,v $
 *        $State: Exp $
 *
 *     $Revision: 2.0 $
 *       $Author: anidel $
 *         $Date: 1998/10/22 08:38:47 $
 *       $Locker:  $
 */              
/* -+-_== */ 

/* ./src/crypt/arithmetic/config.h */

/* 
 *
 * $Id: config.h,v 2.0 1998/10/22 08:38:47 anidel Exp $
 *
 * $Log: config.h,v $
 * Revision 2.0  1998/10/22 08:38:47  anidel
 * Initial revision
 *
 * Revision 2.0  1998/10/22 08:38:29  anidel
 * Initial revision
 *
 * Revision 1.2  1998/01/19 14:49:04  anidel
 * *** empty log message ***
 *
 * Revision 1.1  1997/12/12 00:05:12  anidel
 * Initial revision
 *
 *
 */
 
/* Dummy-File */
